﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace VoltageInput_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
		formCleaner cleaner;
		VoltageInput vin = null;
		private ErrorEventBox errorBox;

		public Form1() {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			open.commandLine = commandLine;
			InitializeComponent();
		}

		protected override bool ProcessDialogKey(Keys keyData) {
			if (keyData == (Keys.W | Keys.Control)) {
				this.Close();
				return true;
			} else
				return base.ProcessDialogKey(keyData);
		}

		private void Form1_Load(object sender, EventArgs e) {
			Clear();

			

			errorBox = new ErrorEventBox(this);

			vin = new VoltageInput();

			vin.Attach += vin_Attach;
			vin.Detach += vin_Detach;
			vin.Error += vin_Error;
			vin.VoltageChange += vin_VoltageChange;
			vin.SensorChange += Vin_SensorChange;

            commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

            try
            { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
                vin.Channel = phidgetParameters.Channel; //selects the channel on the device to open
                vin.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
                vin.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
                vin.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

                if (phidgetParameters.isRemote) //are we trying to open a remote device?
                {
                    vin.IsRemote = true;
                    Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
                    if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
                        Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
                }
                else
                    vin.IsLocal = true;

                vin.Open(); //open the device specified by the above parameters
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
        }


		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			vin.Close();
		}

		void Clear() {
			phidgetInfoBox.Clear();

			settingsBox.Visible = false;

			voltage_range_panel.Visible = false;
			dataBox.Visible = false;
			sensorTypeLbl.Visible = false;
			sensorTypeCmb.Visible = false;
			sensor_value_panel.Visible = false;

			PowerSupplyPanel.Visible = false;
		}

		void vin_Attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox.FillPhidgetInfo((Phidget)sender);

			VoltageInput attachedDevice = (VoltageInput)sender;
			this.sensorTypeCmb.SelectedIndexChanged -= sensorTypeCmb_SelectedIndexChanged;
			this.voltageRangeCmb.SelectedIndexChanged -= voltageRangeCmb_SelectedIndexChanged;

			try {
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;

				//differing max/min means the interval is a settable configuration
				if (attachedDevice.MaxDataInterval == attachedDevice.MinDataInterval) {
					dataIntervalTrk.Enabled = false;
				} else {
					dataIntervalTrk.Enabled = true;
					dataIntervalTrk.Value = attachedDevice.DataInterval;
				}
			} catch (PhidgetException ex) { errorBox.addMessage("Error setting interval: " + ex.Message); }

			try {
				changeTriggerTrk.Unit = "V";
				changeTriggerTrk.minTxt = attachedDevice.MinVoltageChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinVoltageChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxVoltageChangeTrigger;
				changeTriggerTrk.maxTxt = attachedDevice.MaxVoltageChangeTrigger.ToString();


				//differing max/min means the trigger is a settable configuration
				if (attachedDevice.MaxVoltageChangeTrigger == attachedDevice.MinVoltageChangeTrigger) {
					changeTriggerTrk.Enabled = false;
				} else {
					changeTriggerTrk.Value = attachedDevice.VoltageChangeTrigger;
				}
			} catch (PhidgetException ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }

			// Set up form according to device. Note that we set combo box values, which set the Phidget properties in the combo box change events.
			Enum[] supportedVoltageRanges;
			switch (vin.DeviceID) {

			case DeviceID.PN_ADP1000:
				voltage_range_panel.Visible = true;
				supportedVoltageRanges = new Enum[] {VoltageRange.MilliVolts_400,
														 VoltageRange.Volts_2};
				voltageRangeCmb.DataSource = supportedVoltageRanges.ToList();
				voltageRangeCmb.SelectedValue = attachedDevice.VoltageRange;
				break;

			case DeviceID.PN_VCP1000:
				voltage_range_panel.Visible = true;
				supportedVoltageRanges = new Enum[] {VoltageRange.MilliVolts_312_5,
														 VoltageRange.Volts_40};
				voltageRangeCmb.DataSource = supportedVoltageRanges.ToList();
				voltageRangeCmb.SelectedValue = attachedDevice.VoltageRange;
				break;

			case DeviceID.PN_VCP1001:
				voltage_range_panel.Visible = true;
				supportedVoltageRanges = new Enum[] {VoltageRange.Volts_5,
														 VoltageRange.Volts_15,
														 VoltageRange.Volts_40,
														 VoltageRange.Auto};
				voltageRangeCmb.DataSource = supportedVoltageRanges.ToList();
				voltageRangeCmb.SelectedValue = attachedDevice.VoltageRange;
				break;

			case DeviceID.PN_VCP1002:
				voltage_range_panel.Visible = true;
				supportedVoltageRanges = new Enum[] {VoltageRange.MilliVolts_10,
														 VoltageRange.MilliVolts_40,
														 VoltageRange.MilliVolts_200,
														 VoltageRange.MilliVolts_1000,
														 VoltageRange.Auto};
				voltageRangeCmb.DataSource = supportedVoltageRanges.ToList();
				voltageRangeCmb.SelectedValue = attachedDevice.VoltageRange;
				break;

			case DeviceID.PN_DAQ1400:
				Enum[] supportedPowerSupplies;
				PowerSupplyPanel.Visible = true;
				supportedPowerSupplies = new Enum[] {PowerSupply.Off,
														PowerSupply.Volts_12,
														PowerSupply.Volts_24};
				powerSupplyCmb.DataSource = supportedPowerSupplies.ToList();
				powerSupplyCmb.SelectedValue = attachedDevice.PowerSupply;
				break;

			case DeviceID.PN_1058:
				break;

			default:  // Standard 5V Sensor Ports
				if (attachedDevice.ChannelSubclass == ChannelSubclass.VoltageInputSensorPort) {
					sensorTypeLbl.Visible = true;
					sensor_value_panel.Visible = true;
					sensorTypeCmb.DataSource = typeof(VoltageSensorType).ToList();
					sensorTypeCmb.SelectedValue = attachedDevice.SensorType;
					sensorTypeCmb.Visible = true;
				}
				break;
			}

			this.voltageRangeCmb.SelectedIndexChanged += voltageRangeCmb_SelectedIndexChanged;
			this.sensorTypeCmb.SelectedIndexChanged += sensorTypeCmb_SelectedIndexChanged;
			settingsBox.Visible = true;
			dataBox.Visible = true;
		}

		void vin_Detach(object sender, Phidget22.Events.DetachEventArgs e) {
			Clear();
			cleaner.clean();
		}

		void vin_Error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		void vin_VoltageChange(object sender, Phidget22.Events.VoltageInputVoltageChangeEventArgs e) {
			voltageTxt.Text = vin.Voltage.ToString() + " V";
		}
		private void Vin_SensorChange(object sender, VoltageInputSensorChangeEventArgs e) {
			sensorValueTxt.Text = e.SensorValue.ToString() + " " + e.SensorUnit.Symbol;
		}

		private void sensorTypeCmb_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				vin.SensorType = (VoltageSensorType)Enum.Parse(typeof(VoltageSensorType), ((ComboBox)sender).SelectedValue.ToString());
			} catch (Exception ex) {
				errorBox.addMessage("Error setting sensor type: " + ex.Message);
			}

			if (sensorTypeCmb.SelectedIndex == 0) {
				label5.Enabled = true; //standard voltage ratio events (i.e. generic sensor type selected)
				voltageTxt.Enabled = true;
				sensor_value_panel.Enabled = false;
			} else {
				label5.Enabled = false; //specific event for sensor type (this disables normal events)
				voltageTxt.Enabled = false;
				sensor_value_panel.Enabled = true;
			}
		}

		private void voltageRangeCmb_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				vin.VoltageRange = (VoltageRange)Enum.Parse(typeof(VoltageRange), ((ComboBox)sender).SelectedValue.ToString());
			} catch (Exception ex) {
				errorBox.addMessage("Error setting voltage range: " + ex.Message);
			}
		}

		private void dataIntervalTrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					vin.DataInterval = (int)dataIntervalTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
		}
		private void dataIntervalTrk_MouseUp(object sender, MouseEventArgs e) {
			try {
				vin.DataInterval = (int)dataIntervalTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting data interval: " + ex.Message);
			}
		}


		private void changeTriggertrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					vin.VoltageChangeTrigger = changeTriggerTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}
		}

		private void changeTriggertrk_MouseUp(object sender, MouseEventArgs e) {
			try {
				vin.VoltageChangeTrigger = changeTriggerTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting change trigger: " + ex.Message);
			}
		}

		private void powerSupplyCmb_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				vin.PowerSupply = (PowerSupply)Enum.Parse(typeof(PowerSupply), ((ComboBox)sender).SelectedValue.ToString());
			} catch (Exception ex) {
				errorBox.addMessage("Error setting voltage range: " + ex.Message);
			}
		}
	}
}
